// Copyright (c) 2022, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
// All rights reserved.
//
// This file is licensed according to the BSD 3-clause license as follows:
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the "Linz Center of Mechatronics GmbH" and "LCM" nor
//       the names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
// IN NO EVENT SHALL "Linz Center of Mechatronics GmbH" BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// This file is part of X2C. http://x2c.lcm.at/
// $LastChangedRevision: 1603 $
// $LastChangedDate:: 2019-01-21 19:02:13 +0100#$
//
// Function to generate the Scilab help for all libraries
// Parameters:
// none

function [] = createAllLibraryHelps()

jimport at.lcm.x2c.utils.LibraryUtils;

// get system architecture
arch = getArchitecture();

libs_root = jinvoke(jinvoke(LibraryUtils, "getLibraryRootDirectory"), "toString");

X2CLibraryPaths = [];
libs = ls(libs_root)';
for curLib=libs
    if isfile(fullfile(libs_root, curLib, "x2c_library.xml")) then
        X2CLibraryPaths = [X2CLibraryPaths, curLib];
    end
end

mprintf("++ Creation of library help files\n")
try
    for curLib = X2CLibraryPaths
        scilabDir = getLibraryScilabPath(fullfile(libs_root, curLib));
        if isdir(scilabDir) then
            // only remove files in existing directories
            if  strcmp(curLib, "Basic") then
                // ignore Basic library
                generateLibraryHelp(curLib);
                helpDir = fullfile(scilabDir, "help");
                if isdir(helpDir) then
                    exec(fullfile(helpDir, 'builder_help.sce'), -1);
                end 
            else
                // do nothing for Basic library
            end
        end
    end
    mprintf("-- Done\n");
catch
    mprintf("-- FAILED\n");
    mprintf("ERROR: %s\n\n", lasterror());
end

endfunction
